# Generated from ../flash_patcher/antlr_source/AssetPack.g4 by ANTLR 4.13.1
from antlr4 import *
if "." in __name__:
    from .AssetPackParser import AssetPackParser
else:
    from AssetPackParser import AssetPackParser

# This class defines a complete listener for a parse tree produced by AssetPackParser.
class AssetPackListener(ParseTreeListener):

    # Enter a parse tree produced by AssetPackParser#addAssetBlock.
    def enterAddAssetBlock(self, ctx:AssetPackParser.AddAssetBlockContext):
        pass

    # Exit a parse tree produced by AssetPackParser#addAssetBlock.
    def exitAddAssetBlock(self, ctx:AssetPackParser.AddAssetBlockContext):
        pass


    # Enter a parse tree produced by AssetPackParser#root.
    def enterRoot(self, ctx:AssetPackParser.RootContext):
        pass

    # Exit a parse tree produced by AssetPackParser#root.
    def exitRoot(self, ctx:AssetPackParser.RootContext):
        pass


    # Enter a parse tree produced by AssetPackParser#file_name.
    def enterFile_name(self, ctx:AssetPackParser.File_nameContext):
        pass

    # Exit a parse tree produced by AssetPackParser#file_name.
    def exitFile_name(self, ctx:AssetPackParser.File_nameContext):
        pass



del AssetPackParser