# Generated from ../flash_patcher/antlr_source/PatchfileParser.g4 by ANTLR 4.13.1
from antlr4 import *
if "." in __name__:
    from .PatchfileParser import PatchfileParser
else:
    from PatchfileParser import PatchfileParser

# This class defines a complete listener for a parse tree produced by PatchfileParser.
class PatchfileParserListener(ParseTreeListener):

    # Enter a parse tree produced by PatchfileParser#addBlockHeader.
    def enterAddBlockHeader(self, ctx:PatchfileParser.AddBlockHeaderContext):
        pass

    # Exit a parse tree produced by PatchfileParser#addBlockHeader.
    def exitAddBlockHeader(self, ctx:PatchfileParser.AddBlockHeaderContext):
        pass


    # Enter a parse tree produced by PatchfileParser#addBlock.
    def enterAddBlock(self, ctx:PatchfileParser.AddBlockContext):
        pass

    # Exit a parse tree produced by PatchfileParser#addBlock.
    def exitAddBlock(self, ctx:PatchfileParser.AddBlockContext):
        pass


    # Enter a parse tree produced by PatchfileParser#addBlockText.
    def enterAddBlockText(self, ctx:PatchfileParser.AddBlockTextContext):
        pass

    # Exit a parse tree produced by PatchfileParser#addBlockText.
    def exitAddBlockText(self, ctx:PatchfileParser.AddBlockTextContext):
        pass


    # Enter a parse tree produced by PatchfileParser#removeBlock.
    def enterRemoveBlock(self, ctx:PatchfileParser.RemoveBlockContext):
        pass

    # Exit a parse tree produced by PatchfileParser#removeBlock.
    def exitRemoveBlock(self, ctx:PatchfileParser.RemoveBlockContext):
        pass


    # Enter a parse tree produced by PatchfileParser#replaceNthBlockHeader.
    def enterReplaceNthBlockHeader(self, ctx:PatchfileParser.ReplaceNthBlockHeaderContext):
        pass

    # Exit a parse tree produced by PatchfileParser#replaceNthBlockHeader.
    def exitReplaceNthBlockHeader(self, ctx:PatchfileParser.ReplaceNthBlockHeaderContext):
        pass


    # Enter a parse tree produced by PatchfileParser#replaceNthBlock.
    def enterReplaceNthBlock(self, ctx:PatchfileParser.ReplaceNthBlockContext):
        pass

    # Exit a parse tree produced by PatchfileParser#replaceNthBlock.
    def exitReplaceNthBlock(self, ctx:PatchfileParser.ReplaceNthBlockContext):
        pass


    # Enter a parse tree produced by PatchfileParser#replaceAllBlock.
    def enterReplaceAllBlock(self, ctx:PatchfileParser.ReplaceAllBlockContext):
        pass

    # Exit a parse tree produced by PatchfileParser#replaceAllBlock.
    def exitReplaceAllBlock(self, ctx:PatchfileParser.ReplaceAllBlockContext):
        pass


    # Enter a parse tree produced by PatchfileParser#replaceBlockText.
    def enterReplaceBlockText(self, ctx:PatchfileParser.ReplaceBlockTextContext):
        pass

    # Exit a parse tree produced by PatchfileParser#replaceBlockText.
    def exitReplaceBlockText(self, ctx:PatchfileParser.ReplaceBlockTextContext):
        pass


    # Enter a parse tree produced by PatchfileParser#root.
    def enterRoot(self, ctx:PatchfileParser.RootContext):
        pass

    # Exit a parse tree produced by PatchfileParser#root.
    def exitRoot(self, ctx:PatchfileParser.RootContext):
        pass


    # Enter a parse tree produced by PatchfileParser#function.
    def enterFunction(self, ctx:PatchfileParser.FunctionContext):
        pass

    # Exit a parse tree produced by PatchfileParser#function.
    def exitFunction(self, ctx:PatchfileParser.FunctionContext):
        pass


    # Enter a parse tree produced by PatchfileParser#lineNumber.
    def enterLineNumber(self, ctx:PatchfileParser.LineNumberContext):
        pass

    # Exit a parse tree produced by PatchfileParser#lineNumber.
    def exitLineNumber(self, ctx:PatchfileParser.LineNumberContext):
        pass


    # Enter a parse tree produced by PatchfileParser#end.
    def enterEnd(self, ctx:PatchfileParser.EndContext):
        pass

    # Exit a parse tree produced by PatchfileParser#end.
    def exitEnd(self, ctx:PatchfileParser.EndContext):
        pass



del PatchfileParser